//+------------------------------------------------------------------+
//|                                            HST_volume_change.mq4 |
//|                                         Copyright  2009, gordon |
//|                                 http://www.mql4.com/users/gordon |
//+------------------------------------------------------------------+
#property copyright     "Copyright  2009, gordon"
#property link          "http://www.mql4.com/users/gordon"
#property show_inputs

#include <stdlib.mqh>
#include <stderror.mqh>

#define HEADER_SIZE     148         // size of HST file header
#define DATA_SIZE       36          // size of datetime & O/C/H/L data
#define VOLUME_SIZE     8           // size of volume info

// functions for finding files in directory                                
#import "kernel32.dll"                                  
   int  FindFirstFileA( string path, int& answer[] );
   bool FindNextFileA( int handle, int& answer[] );
   bool FindClose( int handle );
#import

extern double set_volume = 4.0;
//+------------------------------------------------------------------+
//| script program start function                                    |
//+------------------------------------------------------------------+
int start()
{
   string path = TerminalPath() + "\experts\files";
   int answer[80], 
       handle = FindFirstFileA( path + "\*.hst", answer );
   
   // extract first file name and process it
   string file_name = ExtractName( answer );
   ProcessHST( file_name, set_volume );   
   ArrayInitialize( answer, 0 );
   
   // loop on all 'Next' files and process each of them
   while( FindNextFileA( handle, answer ) ) {
      file_name = ExtractName( answer );
      ArrayInitialize( answer, 0 );
      ProcessHST( file_name, set_volume );  
   }
   
   // close find handle and exit
   FindClose( handle );
   return(0);
}
//+------------------------------------------------------------------+
//| extract file name from answer returned by FindFirst/NextFileA()  |
//| source: http://articles.mql4.com/723                             |
//+------------------------------------------------------------------+
string ExtractName( int answer[] )
{
   string name = "";
   
   //name is contained in element 11 to 76 (include) in answer[]
   for ( int i = 11; i < 64+11; i++ ) {
      int curr = answer[i];
      name = name + CharToStr( curr       & 0x000000FF )       
                  + CharToStr( curr >> 8  & 0x000000FF )
                  + CharToStr( curr >> 16 & 0x000000FF )
                  + CharToStr( curr >> 24 & 0x000000FF );
   }
   return(name);
} 
//+------------------------------------------------------------------+
//| process HST file, set volume to passed volume                    |
//+------------------------------------------------------------------+
void ProcessHST( string file_name, double volume )
{
   Print( "Start processing HST file: ", file_name );
   
   int handle = FileOpen( file_name, FILE_BIN|FILE_READ|FILE_WRITE );
   if ( handle < 0 ) {
      Print( "Failed to open HST file ", file_name, 
             ", due to error", ErrorText(GetLastError()) );
      return;
   }
   
   //we get file size and set byte counter to file size
   int file_size = FileSize(handle), 
       bytes_cnt = file_size;
   
   // seek to end of header (start of data)
   if ( !FileSeek( handle, HEADER_SIZE, SEEK_SET ) ) {
      Print( "Failed to seek, HST file ", file_name,
             ", due to error", ErrorText(GetLastError()) );
      FileClose(handle);
      return;
   } else {
      bytes_cnt -= HEADER_SIZE;
   }
   
   // loop on all volume values and overwrite them
   while ( bytes_cnt > 1 ) {
      // seek to volume value (skip date & O/C/H/L)      
      if ( !FileSeek( handle, DATA_SIZE, SEEK_CUR ) ) {
         Print( "Failed to seek, HST file ", file_name, 
                ", due to error", ErrorText(GetLastError()) );
         FileClose(handle);
         return;
      } else {
         bytes_cnt -= DATA_SIZE;
      }
      
      //write volume info and print error if occured
      int res = FileWriteDouble( handle, volume );
      if ( res < 0 ) {
         Print( "Failed to write volume value, HST file ", file_name,
                ", due to error", ErrorText(GetLastError()) );
         FileClose(handle);
         return;
      } else {
         bytes_cnt -= VOLUME_SIZE;
      }
   }     // end while
   
   // verify file size is the same (for debugging) and print success
   if ( file_size == FileSize(handle) ) {
      Print( "HST file ", file_name, " processed successfully!" );
   } else {
      Print( "File size missmatch, file ", file_name, " failed!" );
   }
   
   FileClose(handle);
   return;
}
//+------------------------------------------------------------------+
//| return error number and description                              |
//+------------------------------------------------------------------+
string ErrorText( int err )
{
	return( "" + err + ": " + ErrorDescription(err) );
}
//+------------------------------------------------------------------+

